package org.beigesoft.web.service;

/*
 * Copyright (c) 2018 Beigesoft ™
 *
 * Licensed under the GNU General Public License (GPL), Version 2.0
 * (the "License");
 * you may not use this file except in compliance with the License.
 *
 * You may obtain a copy of the License at
 *
 * http://www.gnu.org/licenses/old-licenses/gpl-2.0.en.html
 */

import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpSession;

import org.beigesoft.service.ISrvI18n;
import org.beigesoft.log.ILogger;

/**
 * <p>Service that tracks sessions.</p>
 * @author Yury Demidenko
 */
public class SessionTracker implements ISessionTracker {

  /**
   * <p>Logger.</p>
   **/
  private ILogger logger;

  /**
   * <p>I18N service.</p>
   **/
  private ISrvI18n srvI18n;

  /**
   * <p>Tracks new session if need.</p>
   * @param pReq HTTP request
   * @throws Exception - an exception
   **/
  @Override
  public final void trackNewIfNeed(
    final HttpServletRequest pReq) throws Exception {
    HttpSession session = pReq.getSession();
    if (session.getAttribute("isNewSessionTracked") == null) {
      // scamware can make any headers, so user-agent is not informative
      String msg = "New session from IP/port/host: " + pReq.getRemoteAddr()
        + "/" + pReq.getRemotePort() + "/" + pReq.getRemoteHost();
      this.logger.info(null, SessionTracker.class, msg);
      session.setAttribute("isNewSessionTracked", Boolean.TRUE);
    }
  }

  /**
   * <p>Tracks login fail.</p>
   * @param pReq HTTP request
   * @throws Exception - an exception
   **/
  @Override
  public final void trackLoginFail(
    final HttpServletRequest pReq) throws Exception {
    HttpSession session = pReq.getSession();
    Integer flCountAttempt = (Integer) session.getAttribute("flCountAttempt");
    if (flCountAttempt == null) {
      flCountAttempt = 0;
    }
    flCountAttempt++;
    session.setAttribute("flCountAttempt", flCountAttempt);
    if (flCountAttempt < 6) {
      pReq.setAttribute("loginErrorJsp",
        srvI18n.getMsg("invalid_user_name_or_password"));
      // scamware can make any headers, so user-agent is not informative
      String msg = "Fail login attempt from IP/port/host/attempt: "
        + pReq.getRemoteAddr() + "/" + pReq.getRemotePort()
          + "/" + pReq.getRemoteHost() + "/" + flCountAttempt;
      logger.warn(null, SessionTracker.class, msg);
    } else {
      // banned for web.xml session-timeout minits
      session.setAttribute("isBanned", Boolean.TRUE);
      pReq.setAttribute("loginBanJsp", this.srvI18n.getMsg("loginBan"));
      String msg = "Ban login attempt from IP/port/host/attempt: "
        + pReq.getRemoteAddr() + "/" + pReq.getRemotePort()
          + "/" + pReq.getRemoteHost() + "/" + flCountAttempt;
      logger.error(null, SessionTracker.class, msg);
    }
  }

  //SGS:
  /**
   * <p>Getter for logger.</p>
   * @return ILogger
   **/
  public final ILogger getLogger() {
    return this.logger;
  }

  /**
   * <p>Setter for logger.</p>
   * @param pLogger reference
   **/
  public final void setLogger(final ILogger pLogger) {
    this.logger = pLogger;
  }

  /**
   * <p>Getter for srvI18n.</p>
   * @return ISrvI18n
   **/
  public final ISrvI18n getSrvI18n() {
    return this.srvI18n;
  }

  /**
   * <p>Setter for srvI18n.</p>
   * @param pSrvI18n reference
   **/
  public final void setSrvI18n(final ISrvI18n pSrvI18n) {
    this.srvI18n = pSrvI18n;
  }
}
